import MenuTabs from './components/MenuTabs'
import Page from './pages/Page'
import User from './pages/User'
import WebPage from './pages/WebPage'
import ScoreMapPage from './pages/ScoreMapPage'
import React, { useState } from 'react'
import { IonApp, IonRouterOutlet, IonSplitPane } from '@ionic/react'
import { IonReactRouter } from '@ionic/react-router'
import { Redirect, Route } from 'react-router-dom'
import UserContext from './context/UserContext'
import AppContext from './context/AppContext'
import ScoreContext from './context/ScoreContext'
import socialMediaFeeds from './resources/socialMediaFeeds.json'
import ls from 'local-storage'

/* Core CSS required for Ionic components to work properly */
import '@ionic/react/css/core.css';

/* Basic CSS for apps built with Ionic */
import '@ionic/react/css/normalize.css';
import '@ionic/react/css/structure.css';
import '@ionic/react/css/typography.css';

/* Optional CSS utils that can be commented out */
import '@ionic/react/css/padding.css';
import '@ionic/react/css/float-elements.css';
import '@ionic/react/css/text-alignment.css';
import '@ionic/react/css/text-transformation.css';
import '@ionic/react/css/flex-utils.css';
import '@ionic/react/css/display.css';

/* Theme variables */
import './theme/variables.css';

const App: React.FC = () => {

  // User state (stored in context)
  const [username, setUsername] = useState("")
  const [email, setEmail] = useState("")
  const [firstname, setFirstname] = useState("")
  const [fullname, setFullname] = useState("")
  const [useData, setUseData] = useState(false)
  const [interestMoths, setInterestMoths] = useState(false)
  const [interestButterflies, setInterestButterflies] = useState(false)
  const [interestGrasshoppers, setInterestGrasshoppers] = useState(false)
  const [tool, setTool] = useState("")
  const [path, setPath] = useState("")
  const [showOpts, setShowOpts] = useState(false)
  const [generalConsent, setGeneralConsent] = useState(false)
  const [commsConsent, setCommsConsent] = useState('')
  const [otherPlatformConsent, setOtherPlatformConsent] = useState(false)
  const [ispotUsername, setIspotUsername] = useState("")
  const [irecordUsername, setIrecordUsername] = useState("")
  

  // Score map options
  const [showRowAccess, setShowRowAccess] = useState(false)
  const [showPointAccess, setShowPointAccess] = useState(false)
  const [showGreenspaceAccess, setShowGreenspaceAccess] = useState(true)
  const [showOpenAccess, setShowOpenAccess] = useState(false)
  const [showNtAccess, setShowNtAccess] = useState(false)
  const [showLondonAccess, setShowLondonAccess] = useState(false)
  const [showDecideScoreLayer, setShowDecideScoreLayer] = useState(true)
  const [opacityDecideScoreLayer, setOpacityDecideScoreLayer] = useState(0.6)
  const [scaleDecideScoreLayer, setScaleDecideScoreLayer] = useState(true)
  const [thresholdDecideScoreLayer, setThresholdDecideScoreLayer] = useState(0)
  const [radiusDecideScore, setRadiusDecideScore] = useState(5000)
  const [scoreTaxa, setScoreTaxa] = useState(["butterfly"])
  const [scoreColour, setScoreColour] = useState("viridis_truncated")

  let lsSmfeed: string = String(ls('smfeed')) 
  lsSmfeed = lsSmfeed !== 'null' ? lsSmfeed : socialMediaFeeds[0].id
  const [mediaFeed, setMediaFeed] = useState(lsSmfeed)

  const user = {
    username: username,
    setUsername: setUsername,
    email: email,
    setEmail: setEmail,
    firstname: firstname,
    setFirstname: setFirstname,
    fullname: fullname,
    setFullname: setFullname,
    mediaFeed: mediaFeed,
    setMediaFeed: setMediaFeed,
    useData: useData,
    setUseData: setUseData,
    interestMoths: interestMoths,
    setInterestMoths: setInterestMoths,
    interestButterflies: interestButterflies,
    setInterestButterflies: setInterestButterflies,
    interestGrasshoppers:  interestGrasshoppers,
    setInterestGrasshoppers: setInterestGrasshoppers,
    generalConsent: generalConsent,
    setGeneralConsent: setGeneralConsent,
    commsConsent: commsConsent,
    setCommsConsent: setCommsConsent,
    otherPlatformConsent: otherPlatformConsent,
    setOtherPlatformConsent: setOtherPlatformConsent,
    ispotUsername: ispotUsername,
    setIspotUsername: setIspotUsername,
    irecordUsername: irecordUsername,
    setIrecordUsername: setIrecordUsername,
  }

  const app = {
    tool: tool,
    setTool: setTool,
    path: path,
    setPath: setPath,
    showOpts: showOpts,
    setShowOpts: setShowOpts,
    showRowAccess: showRowAccess,
    setShowRowAccess: setShowRowAccess,
    showPointAccess: showPointAccess,
    setShowPointAccess: setShowPointAccess,
    showGreenspaceAccess: showGreenspaceAccess,
    setShowGreenspaceAccess: setShowGreenspaceAccess,
    showOpenAccess: showOpenAccess,
    setShowOpenAccess: setShowOpenAccess,
    showNtAccess: showNtAccess,
    setShowNtAccess: setShowNtAccess,
    showLondonAccess: showLondonAccess,
    setShowLondonAccess: setShowLondonAccess,
    showDecideScoreLayer: showDecideScoreLayer,
    setShowDecideScoreLayer: setShowDecideScoreLayer,
    opacityDecideScoreLayer: opacityDecideScoreLayer,
    setOpacityDecideScoreLayer: setOpacityDecideScoreLayer,
    scaleDecideScoreLayer: scaleDecideScoreLayer,
    setScaleDecideScoreLayer: setScaleDecideScoreLayer,
    thresholdDecideScoreLayer: thresholdDecideScoreLayer,
    setThresholdDecideScoreLayer: setThresholdDecideScoreLayer,
    radiusDecideScore: radiusDecideScore,
    setRadiusDecideScore: setRadiusDecideScore,
    scoreTaxa: scoreTaxa,
    setScoreTaxa: setScoreTaxa,
    scoreColour: scoreColour,
    setScoreColour: setScoreColour,
    radiusMin: 1000,
    radiusMax: 30000,
    surveyLink: 'https://ceh-online-surveys.onlinesurveys.ac.uk/decide_feedbackform_intool_general'
  }

  const [scoreImageUrl, setScoreImageUrl] = useState('')
  const [prowGeojson, setProwGeojson] = useState('')
  const [greenspaceGeojson, setGreenspaceGeojson] = useState('')
  const [accesspointGeojson, setAccesspointGeojson] = useState('')
  const [accesslandGeojson, setAccesslandGeojson] = useState('')
  const [ntGeojson, setNtGeojson] = useState('')
  const [londonGeojson, setLondonGeojson] = useState('')
  const [radius, setRadius] = useState(5000)
  const [lat, setLat] = useState(0)
  const [lon, setLon] = useState(0)

  const score = {
    scoreImageUrl: scoreImageUrl,
    setScoreImageUrl: setScoreImageUrl,
    prowGeojson: prowGeojson,
    setProwGeojson: setProwGeojson,
    greenspaceGeojson: greenspaceGeojson,
    setGreenspaceGeojson: setGreenspaceGeojson,
    accesspointGeojson: accesspointGeojson,
    setAccesspointGeojson: setAccesspointGeojson,
    accesslandGeojson: accesslandGeojson,
    setAccesslandGeojson: setAccesslandGeojson,
    ntGeojson: ntGeojson,
    setNtGeojson: setNtGeojson,
    londonGeojson: londonGeojson,
    setLondonGeojson: setLondonGeojson,
    radius: radius,
    setRadius: setRadius,
    lat: lat,
    setLat: setLat,
    lon: lon,
    setLon: setLon,
  }
  

  return (
    <AppContext.Provider value={app}>
      <UserContext.Provider value={user}>
        <ScoreContext.Provider value={score}>
          <IonApp>
            <IonReactRouter>
              <IonSplitPane contentId="main">
                <MenuTabs />
                <IonRouterOutlet id="main">
                  <Route path="/user" component={User} />
                  <Route path="/(app|opts)/scoremap/" component={ScoreMapPage} />
                  <Route path="/app/page/:name" component={Page} />
                  <Route path="/info/:page" component={WebPage} />
                  <Redirect from="/" to="/info/decide_info" exact />
                </IonRouterOutlet>
              </IonSplitPane>
            </IonReactRouter>
          </IonApp>
        </ScoreContext.Provider>
      </UserContext.Provider>
    </AppContext.Provider>
  );
};

export default App;
