import React, { FunctionComponent, useContext } from 'react'
import ScoreContext from '../context/ScoreContext'
import L from 'leaflet'
import AppContext from '../context/AppContext'
import AccessGeojsonLayer from './AccessGeojsonLayer'

interface AccessGeojsonProps {
}

const AccessGeojsonLayers: FunctionComponent<AccessGeojsonProps> = () => {
  const { 
    prowGeojson,
    setProwGeojson,
    greenspaceGeojson,
    setGreenspaceGeojson,
    accesspointGeojson,
    setAccesspointGeojson,
    accesslandGeojson,
    setAccesslandGeojson,
    ntGeojson,
    setNtGeojson,
    londonGeojson,
    setLondonGeojson
  } = useContext(ScoreContext)
  const { showRowAccess, showPointAccess, showGreenspaceAccess, showOpenAccess, showNtAccess, showLondonAccess} = useContext(AppContext)
  return (<>
    <AccessGeojsonLayer type='accessland'
      display={showOpenAccess}
      //zIndex={497}
      lkey={Math.random()} // Key is always changed (random number) to ensure correct ordering of geojson layers
      geojson={accesslandGeojson}
      setGeojson={setAccesslandGeojson}
      pathOpts={{
        color: '#AAAAAA'
      }}
      pointToLayer={null}
      onEachFeature={function(feature: any, layer: any) {
        let html = '<b>Open access</b><br/>'
        if (feature.properties['Descrip']) {
          //html = `${html}<b>Description:</b> ${feature.properties['Descrip']}`
        }
        layer.bindPopup(html)
      }}
    />
    <AccessGeojsonLayer type='greenspace'
      display={showGreenspaceAccess}
      //zIndex={498}
      key={Math.random()}
      geojson={greenspaceGeojson}
      setGeojson={setGreenspaceGeojson}
      pathOpts={{
        color: '#888888'
      }}
      pointToLayer={null}
      onEachFeature={function(feature: any, layer: any) {
        let html = '<b>Greenspace</b><br/>'
        if (feature.properties['function_']) {
          html = `${html}<b>Function:</b> ${feature.properties['function_']}<br/>`
        }
        if (feature.properties['distName1']) {
          html = `${html}<b>District:</b> ${feature.properties['distName1']}`
        }
        layer.bindPopup(html)
      }}
    />
    <AccessGeojsonLayer type='nattrust'
      display={showNtAccess}
      //zIndex={499}
      key={Math.random()}
      geojson={ntGeojson}
      setGeojson={setNtGeojson}
      pathOpts={{
        color: '#yellow'
      }}
      pointToLayer={null}
      onEachFeature={function(feature: any, layer: any) {
        let html = '<b>National Trust property</b><br/>'
        if (feature.properties['fclass']) {
          html = `${html}<b>Name:</b> ${feature.properties['Name']}`
        }
        if (feature.properties['trust_type']) {
          html = `${html}<b>Type:</b> ${feature.properties['trust_type'].replace('_', ' ')}`
        }
        layer.bindPopup(html)
      }}
    />
    <AccessGeojsonLayer type='prow'
      display={showRowAccess}
      //zIndex={500}
      key={Math.random()}
      geojson={prowGeojson}
      setGeojson={setProwGeojson}
      pathOpts={{
        color: '#666666'
      }}
      pointToLayer={null}
      onEachFeature={function(feature: any, layer: any) {
        let html = '<b>Public right of way</b><br/>'
        if (feature.properties['ROW_TYPE']) {
          html = `${html}<b>Type:</b> ${feature.properties['ROW_TYPE']}<br/>`
        }
        layer.bindPopup(html)
      }}
    />
    <AccessGeojsonLayer type='london'
      display={showLondonAccess}
      //zIndex={501}
      key={Math.random()}
      geojson={londonGeojson}
      setGeojson={setLondonGeojson}
      pathOpts={{
        color: '#666666'
      }}
      pointToLayer={null}
      onEachFeature={function(feature: any, layer: any) {
        let html = '<b>Right of way</b><br/>'
        if (feature.properties['fclass']) {
          html = `${html}<b>Type:</b> ${feature.properties['fclass']}`
        }
        layer.bindPopup(html)
      }}
    />
    <AccessGeojsonLayer type='accesspoint'
      display={showPointAccess}
      //zIndex={600}
      key={Math.random()}
      geojson={accesspointGeojson}
      setGeojson={setAccesspointGeojson}
      pathOpts={null}
      pointToLayer={function (feature: any, latlng: any) {
        return L.circleMarker(
          latlng, 
          {
            //pane: `access-pane-accesspoint`,
            radius: 4,
            fillColor: "#ff7800",
            color: "#000",
            weight: 1,
            opacity: 1,
            fillOpacity: 0.8
          }
        )
      }}
      onEachFeature={function(feature: any, layer: any) {
        let html = '<b>Point of access</b><br/>'
        if (feature.properties['accessType']) {
          html = `${html}<b>Type:</b> ${feature.properties['accessType']}`
        }
        layer.bindPopup(html)
      }}
    />
  </>) 
}
export default AccessGeojsonLayers
