import { FunctionComponent, useRef, useEffect } from 'react'
//import RL from 'react-leaflet' 
import {useLeafletContext} from '@react-leaflet/core' 
import L from 'leaflet'

// This was experimental - attempting to create a react-leaflet component
// which allows a raster overlay to be clipped to the extents of a geojson layer.
// I was looking at: https://gis.stackexchange.com/questions/353250/clip-boundaries-of-heatmap-according-to-another-feature
// Didn't get very far with it - would need more time, but may be worth doing if client-side clipping is definitely
// required.  

interface ClippedOverlayProps {
  clip: any,
  url: string,
  bounds: any, //LatLngBounds - can't find how to specify LatLngBounds type
  opacity: number
}

const ClippedOverlay: FunctionComponent<ClippedOverlayProps> = (props) => {

  const context: any = useLeafletContext()
  const overlayRef: any = useRef()
  const overlayClippedRef: any = useRef()
  const vectorClipRef: any = useRef()

  const propsRef: any = useRef(props)

  console.log('bounds', props.bounds)
  console.log('bounds', props.clip)

  useEffect(() => {
    const container = context.layerContainer || context.map

    overlayRef.current = L.imageOverlay(props.url, props.bounds, {'opacity': 0})
    container.addLayer(overlayRef.current)

    overlayClippedRef.current = L.imageOverlay(props.url, props.bounds, {'opacity': props.opacity})
    container.addLayer(overlayClippedRef.current)

    vectorClipRef.current =  L.geoJSON(props.clip, {
      //pane: 'clip-pane',
      style: {
        stroke: false,
        fill: true,
        fillColor: '#ffffff',
        fillOpacity: 1
      }
    })
    container.addLayer(vectorClipRef.current)


    // const overlayPane = context.map.getPanes().overlayPane
    // const heatmapCanvas = overlayPane.childNodes[0]
    // heatmapCanvas.style.display = 'none'

    return () => {
      container.removeLayer(overlayRef.current)
      container.removeLayer(overlayClippedRef.current)
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [])

  useEffect(() => {

    console.log('props.bounds', props.bounds)
    console.log('propsRef.current.bounds', propsRef.current.bounds)

    if (props.url !== propsRef.url) {
      overlayRef.current.setUrl(props.url)
    }
    if (props.opacity !== propsRef.opacity) {
      overlayRef.current.setOpacity(props.opacity)
    }
    if (
      props.bounds._northEast.lat !== propsRef.current.bounds._northEast.lat ||
      props.bounds._northEast.lng !== propsRef.current.bounds._northEast.lng ||
      props.bounds._southWest.lat !== propsRef.current.bounds._southWest.lat ||
      props.bounds._southWest.lng !== propsRef.current.bounds._southWest.lng
    ) {
      overlayRef.current.setBounds(props.bounds)
    }
    propsRef.current = props
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [props.url, props.bounds, props.opacity])

  return null
}
export default ClippedOverlay