import { useEffect, useContext } from 'react'
import AppContext from '../context/AppContext'
import './DecideScoreLegend.css'
import L from 'leaflet'
import 'leaflet/dist/leaflet.css'
import * as interpolate from 'color-interpolate'

// This is not typescript because I couldn't find out how to extend Leaflet
// control using typescript.

const DecideScoreLegend = ({map, position, render} ) => {

  // The render argument is simply there to force this control
  // to re-render whenever one of the buttons above it may
  // have changed otherwise their position is swapped.

  const { scoreColour } = useContext(AppContext)
  const { opacityDecideScoreLayer } = useContext(AppContext)

  useEffect(() => {

    //console.log('rendering legend')

    const scoreAsColour = (i) => {
      let colormap
      switch (scoreColour) {
        case 'yellow_red':
          colormap = interpolate(['#ffffcc','#ffeda0','#fed976','#feb24c','#fd8d3c','#fc4e2a','#e31a1c','#bd0026','#800026'])
          break
        case 'viridis_truncated':
          colormap = interpolate(['#440154', '#414487', '#2A788E', '#22A884', '#7AD151', '#FDE725'])
          // In the score server, the viridis_truncated style is calculated like this:
          // cols <- tail(viridis(140), 100)
          // Here we need adjust the score i value to correspond to this truncated scale.
          i = 40/140 + i * 100/140
          //i = 0.3 + i * 0.7
          break
        case 'cividis':
          colormap = interpolate(['white', 'black']) // Would need creating if we ever use.
          break
        default:
          colormap = interpolate(['white', 'black'])
          break
      }
      return colormap(i)
    }

    const legendSwatches = (n) => {
      let html = ''
      for (let i=0; i<n; i++) {
        html = `${html} 
          <div class='swatch' 
            style='
              background-color: ${scoreAsColour(i/(n-1))}; 
              opacity: ${opacityDecideScoreLayer};
              border-radius: ${i===0 ? '0 0 0 4px' : i===n-1 ? '0 0 4px 0' : '0' }
          '></div>`
      }
      return html
    }

    L.Control.DecideLegend = L.Control.extend({
      onAdd: function() {
          var div = L.DomUtil.create('div')
          div.id ='react-score-legend'

          const legend = `<div>
            <div id='react-score-legend-top'>
              <div id='react-score-legend-top-left'>------------------></div>
              <div id='react-score-legend-top-right'>Higher priority for recording</div>
            </div>
            <div id='react-score-legend-bottom'>
              ${legendSwatches(10)}
            </div>
          </div>`

          div.innerHTML  = legend
          return div
      },
      onRemove: function() {
        //console.log('Remove legend')
        // Nothing to do here
      }
    });
    
    L.control.decidelegend = function(opts) {
        return new L.Control.DecideLegend(opts);
    }
    
    const cc = L.control.decidelegend({ position: position }).addTo(map);

    return () => {
      map.removeControl(cc);
      //var div = L.DomUtil.get('react-score-legend')
      //div.innerHTML = 'remove'
      //console.log(div)
    }

  // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [scoreColour, opacityDecideScoreLayer, render])
  return null
}
export default DecideScoreLegend
