import React, { useEffect, FunctionComponent } from 'react'
import './MapButton.css'
import { renderToString } from 'react-dom/server'
import L from 'leaflet'
import 'leaflet/dist/leaflet.css'
import 'leaflet-easybutton'

interface MapButtonProps {
  map: any,
  position?: "topleft" | "topright" | "bottomleft" | "bottomright" | undefined,
  title?: string,
  onClick?: Function,
  id?: string
}
const MapButton: FunctionComponent<MapButtonProps> = ({ children, map, position, title, onClick, id}) => {


  useEffect(() => {

    //console.log('rendering button', id)
    const handleClick = () => {
      if (onClick) {
        onClick()
      }
    }
    const button = L.easyButton ({
      position: position ? position : 'topleft',
      states: [{
        stateName: '',
        title: title ? title: '',
        icon: renderToString(<div className="mapButton">{children}</div>),
        onClick: handleClick
      }],
      id: id ? id : ''
    })
    button.addTo(map)
    return () => {
      // Cleanup
      map.removeControl(button)
      //console.log('remove map button')
    }
  // Took the properties array out so that control is not constantly
  // re-rendered (which changed position in relation to legend control).
  // map, position, title, children, onClick, id
  // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [onClick])
  return null
}
export default MapButton
