import {
  IonRouterOutlet, IonMenu, IonIcon,
  IonTabs, IonTabBar, IonTabButton,
  IonLabel
} from '@ionic/react';
import { 
  menuOutline,
  menuSharp,
  settingsOutline,
  settingsSharp,
  logInOutline, 
  logInSharp, 
  informationCircleOutline, 
  informationCircleSharp,  
  informationOutline, 
  informationSharp,  
  appsOutline, 
  appsSharp, 
  mapOutline, 
  mapSharp,
} from 'ionicons/icons';

// import { IonReactRouter } from '@ionic/react-router';
import {  Route, Switch } from 'react-router-dom'
import React, { useState, useContext } from "react";
import Menu from './Menu';
import Options from '../pages/Options';
import MenuItemsContext, { AppPage } from '../context/MenuItemsContext'
import AppContext from '../context/AppContext'

const MenuTabs: React.FC = () => {

  const [menuItems, setMenuItems]=useState<AppPage[]>([])
  const [visibleSubgroup, setVisibleSubgroup] = useState('')
  const { showOpts, tool } = useContext(AppContext)

  const appPages: AppPage[] = [
    {
      path: 'main-app',
      title: 'App',
      url: '',
      iosIcon: appsOutline,
      mdIcon: appsSharp
    },
    {
      path: 'main-app-map',
      title: 'DECIDE Score Map',
      url: '/opts/scoremap',
      iosIcon: mapOutline,
      mdIcon: mapSharp,
      opts: true
    },
    // {
    //   path: 'main-app-blah',
    //   title: 'Blah',
    //   url: '/app/page/blah',
    //   iosIcon: happyOutline,
    //   mdIcon: happySharp
    // },
    // {
    //   path: 'main-app-blahblah',
    //   title: 'Blah Blah',
    //   url: '/app/page/blahblah',
    //   iosIcon: happyOutline,
    //   mdIcon: happySharp
    // },
    {
      path: 'main-app-login',
      title: 'My account',
      url: '/user',
      iosIcon: logInOutline,
      mdIcon: logInSharp
    },
    { path: 'main-info',
      title: 'Information',
      url: '',
      iosIcon: informationCircleOutline,
      mdIcon: informationCircleSharp
    },
  ]

  if (menuItems.length === 0) {
    fetch(`${process.env.REACT_APP_DBAPI_URL}infopages/menu.txt`,
      {headers : { 
          'Content-Type': 'application/text',
          'Accept': 'application/text'
        }
      }).then(function(response){
        return response.text()
      }).then(function(text) {
        const txt = text.trim()
        let iMenu = 0
        let iItem = 0
        let path = ''
        const currentMenus = ['main-info']
        txt.split('\n').forEach(line => {
          const leadingSpace = line.search(/\S|$/)  // length of leading space
          const splitLine = line.split('##')
          if (splitLine.length === 1) {
            // This is a submenu
            path = `${currentMenus[leadingSpace]}-menu${++iMenu}`
            //currentMenus[leadingSpace+2] = path
            currentMenus[leadingSpace+1] = path
          } else {
            // This is a menu item poiting to a resource
            //console.log(currentMenus)
            path = `${currentMenus[leadingSpace]}-info${++iItem}`
          }
          const item = {
            path: path,
            title: splitLine[0].trim(),
            url: splitLine[1] ? `/info/${splitLine[1].trim()}` : '',
            iosIcon: informationOutline,
            mdIcon: informationSharp,
          }
          appPages.push(item)
        })
        setMenuItems(appPages)
      })
  }
    
  const mi = {
    menuItems: menuItems,
    setMenuItems: setMenuItems,
    visibleSubgroup: visibleSubgroup,
    setVisibleSubgroup: setVisibleSubgroup
  }

  let optsTab
  if (showOpts) {
    optsTab = (
      <IonTabButton tab="tab2" href={`/opts/${tool}`}>
        <IonIcon ios={settingsOutline} md={settingsSharp} />
        <IonLabel>Options</IonLabel>
      </IonTabButton>
    )
  } else {
    optsTab = null
  }

  return (
    <MenuItemsContext.Provider value={mi}>
      <IonMenu contentId="main" type="overlay">
        {/* <IonReactRouter> */}
          <IonTabs>
            <IonRouterOutlet>
              { /* Note can also use regular expression in path spec  */}
              <Switch>
                <Route path="/opts/:tool" component={Options} />
                <Route path={["/menu", "/info", "/app", "/page"]} component={Menu} />
                <Route path="(.*?)" component={Menu} />
              </Switch>
            </IonRouterOutlet>
            <IonTabBar slot="bottom" style={{display: optsTab ? '' : 'none'}} >
              {/* <IonTabButton tab="tab1" href={`${path}`}> */}
              <IonTabButton tab="tab1" href={`/app/${tool}`}>
                <IonIcon ios={menuOutline} md={menuSharp} />
                <IonLabel>Menu</IonLabel>
              </IonTabButton>
              {optsTab}
            </IonTabBar>
          </IonTabs>
        {/* </IonReactRouter> */}
      </IonMenu>
    </MenuItemsContext.Provider>
  );
};
export default MenuTabs;
