import React, { FunctionComponent, useContext, useEffect } from 'react'
import {ImageOverlay, Circle } from 'react-leaflet'
import L from 'leaflet'
import ScoreContext from '../context/ScoreContext'
import AppContext from '../context/AppContext'
import ReactGA from './ReactGAProxy.js'

interface ScoreOverlayProps {
  map: any,
}

let fetching = false

const ScoreOverlay: FunctionComponent<ScoreOverlayProps> = ({ map }) => {
  //const { lat, lon, scoreImageUrl, setScoreImageUrl, radius, setRadius, greenspaceGeojson } = useContext(ScoreContext)
  const { lat, lon, scoreImageUrl, setScoreImageUrl, radius, setRadius } = useContext(ScoreContext)
  const { showDecideScoreLayer, opacityDecideScoreLayer, scaleDecideScoreLayer, thresholdDecideScoreLayer } = useContext(AppContext)
  const { radiusDecideScore } = useContext(AppContext)
  const { scoreTaxa } = useContext(AppContext)
  const { scoreColour } = useContext(AppContext)

  useEffect(() => {
    if (scoreImageUrl==='fetch' && !fetching) {

      ReactGA.event({
        category: 'Decide score map',
        action: 'Fetch Decide score map'
      })

      if (lat) {
        fetching = true
    
        fetch(`${process.env.REACT_APP_SCORE_SERVER_URL}score/point-rad-pre?lat=${lat}&lon=${lon}&rad=${radiusDecideScore}&name=${scoreTaxa[0]}&local=${scaleDecideScoreLayer}&threshold=${thresholdDecideScoreLayer}&colour=${scoreColour}`)
        .then((response) => {
          if (response.status !== 200) {
            throw new Error(String(response.status))
          } else {
            return response.blob()
          }
        }).then((image) => {
          console.log('success')
          if (image) {
            setScoreImageUrl(URL.createObjectURL(image))
          }
        }).catch((error) => {
          console.log('error', error)
          setScoreImageUrl('')
        }).finally(() => {
          fetching = false
          setRadius(radiusDecideScore)
        })
      } else {
        setScoreImageUrl('')
      }
    }
  })

  if (scoreImageUrl && scoreImageUrl !== 'fetch') {
    const circle = L.circle([lat, lon], radius).addTo(map)
    const bounds = circle.getBounds()
    map.removeLayer(circle)
    console.log(scoreImageUrl)

    if (showDecideScoreLayer && scoreImageUrl !== 'fetch') {
      return (<>
        <Circle center={[lat, lon]} radius={radius} pathOptions={{ color: 'black',  fillOpacity:0, weight: 2}} />
        <ImageOverlay url={scoreImageUrl} bounds={bounds} opacity={opacityDecideScoreLayer} />
      </>)
    } else {
      return null
    }
  } else {
    return null
  }
}
export default ScoreOverlay
