const api = async (query, body) => {

  const fetchOpts = {
    method: body ? 'POST' : 'GET',
    headers: {
      'Content-Type': 'application/json'
    }
  }
  if (body) {
    fetchOpts.body = JSON.stringify(body)
  }

  return fetch(`${process.env.REACT_APP_DBAPI_URL}${query}`, fetchOpts).then((response) => {
      if (response.status !== 200) {
        throw new Error(String(response.status))
      } else {
        return response.json()
      }
    }).then((json) => {
      console.log('Request successful', json)
      return json
    }).catch((error) => {
      console.log('error', error)
      return []
    }).finally(() => {
      // Cleanup
    })
}

const setDevUser = async (username) => {
  if (username) {
    return await api(`db/setdevuser/${username}`)
  } else {
    return await api(`db/cleardevuser`)
  }
}

const getUserDetails = async () => {
  return await api(`db/user`)
}

const insertUser = async () => {
  return await api(`db/insert`)
}

const getUsername = async () => {
  return await api('db/username')
}

const updateUser = async (body) => {
  return await api('db/update', body)
}

export { setDevUser, getUsername, getUserDetails, insertUser, updateUser }