import React, { useContext, useEffect } from 'react'
import { IonButtons, IonContent, IonHeader, IonMenuButton, IonPage, IonTitle, IonToolbar } from '@ionic/react'
import { MapContainer, MapConsumer } from 'react-leaflet'
// import { 
//   locationOutline, 
//   locationSharp, 
//   bulbOutline
// } from 'ionicons/icons'
import './ScoreMapPage.css'
import StandardLayers from '../components/StandardLayers'
import StandardMapControls from '../components/StandardMapControls'
import MapButton from '../components/MapButton'
import ScoreOverlay from '../components/ScoreOverlay'
//import AccessGeojson from '../components/AccessGeojson'
import AccessGeojsonLayers from '../components/AccessGeojsonLayers'
import Spinner from '../components/Spinner'
import ScoreContext from '../context/ScoreContext'
import AppContext from '../context/AppContext'
import DecideScoreLegend from '../components/DecideScoreLegend.js'
import Version from '../components/Version'
import Track from '../components/Track'
import ReactGA from '../components/ReactGAProxy.js'
//import forwardLeafletEvents from '../js/leafletEvents'

const ScoreMapPage: React.FC = () => {

  const { setTool, showRowAccess, showPointAccess, showGreenspaceAccess,
    showOpenAccess, showNtAccess, showLondonAccess } = useContext(AppContext)

  //console.log('Rendering scoremap page. showRowAccess', showRowAccess)

  const { setLat, setLon, setScoreImageUrl, setProwGeojson, 
    setGreenspaceGeojson, setAccesspointGeojson, setAccesslandGeojson,
    setNtGeojson, setLondonGeojson } = useContext(ScoreContext)

  useEffect(() => {
    // This has to be done in useEffect (after mounted) to 
    // avoid 'cannot update a component while rendering a 
    // different component' warning .
    setTool('scoremap')
  })

  return (
    <IonPage>
      <Track/>
      <IonHeader>
        <IonToolbar>
          <IonButtons slot="start">
            <IonMenuButton />
          </IonButtons>
          <IonTitle>DECIDE score map (v<Version />)</IonTitle>
          <IonButtons slot="end">          
            <div style={{marginRight: '1em'}}><Spinner /></div>
          </IonButtons>
        </IonToolbar>
      </IonHeader>
      <IonContent>
        {/* <ScoreContext.Provider value={score}> */}
          <MapContainer
            id='decidescoremap'
            preferCanvas={true}
            zoomControl={false}
            attributionControl
            // Centre and zoom to show whole of GB
            center={{ lat: 52.5, lng: -1.5 }}
            zoom={8}
          >
            <MapConsumer>
              {(map) => {
                //forwardLeafletEvents(map)
                return (<>
                  <StandardLayers/>
                  <ScoreOverlay map={map} />
                  <AccessGeojsonLayers/>
                  <MapButton id='current-location-button' map={map} position='topleft' title='Current location' 
                    onClick={function() {
                      ReactGA.event({
                        category: 'Decide score map',
                        action: 'Centre map on current location'
                      })
                      map.locate({setView: true, maxZoom: 16, enableHighAccuracy: true})
                    }}>
                    {/* <IonIcon slot="start" ios={locationOutline} md={locationSharp}  title='Current location' /> */}
                    Get my Current location
                  </MapButton>
                  <MapButton id='decide-score-button' map={map} position='topleft' title='Decide score' 
                    onClick={function () {
                      const centre = map.getCenter()
                      setLat(centre.lat)
                      setLon(centre.lng)
                      setScoreImageUrl("fetch")
                      setProwGeojson(showRowAccess ? "fetch" : "")
                      setGreenspaceGeojson(showGreenspaceAccess ? "fetch" : "")
                      setAccesspointGeojson(showPointAccess ? "fetch" : "")
                      setAccesslandGeojson(showOpenAccess ? "fetch" : "")
                      setNtGeojson(showNtAccess ? "fetch" : "")
                      setLondonGeojson(showLondonAccess ? "fetch" : "")
                    }}>
                    {/* <IonIcon slot="start" ios={bulbOutline} md={bulbOutline}  /> */}
                    Get DECIDE score
                  </MapButton>
                  <DecideScoreLegend map={map} position="topleft" render={Math.random()}/>
                  <StandardMapControls map={map} />
                </>)
              }}
            </MapConsumer>
          </MapContainer>
        {/* </ScoreContext.Provider> */}
      </IonContent>
    </IonPage>
  )
}
export default ScoreMapPage
