import { 
  IonPage, 
  IonContent,
  IonHeader,
  IonToolbar,
  IonButtons,
  IonMenuButton,
  IonTitle,
  // IonGrid,
  // IonRow,
  // IonCol
} from '@ionic/react'
import React, { useState, useEffect } from 'react'
import ReactMarkdown from 'react-markdown/with-html'
import { RouteComponentProps } from 'react-router-dom'
//import SocialMediaFeed from'../components/SocialMediaFeed'
import './WebPage.css'
import Track from '../components/Track'

interface PageProps extends RouteComponentProps<{
  page: string;
}> {}

const WebPage: React.FC<PageProps> = ({match}) => {

  const [markdown, setMarkdown] = useState('')
  //const [mediaFeed, setMediaFeed] = useState('')
  const md = `${process.env.REACT_APP_DBAPI_URL}infopages/md/${match.params.page}.md`

  useEffect(() => {
    async function readMarkdown() {
      let found = false
      await fetch(md, { method: 'HEAD' }).then(res => {found = res.ok})
      if (found){
        await fetch(md).then(res => res.text()).then (text => {
          // Resources in text that contains relative pathnames to image files will not
          // work on localhost because served from a different port, so as a workaround
          // replace for localhost only.
          if (window.location.hostname === 'localhost') {
            text = text.replace(/\/infopages\//g, `${process.env.REACT_APP_DBAPI_URL}infopages/`)
          }
          setMarkdown(text)
        })
      } else {
        setMarkdown('')
      }
    }
    readMarkdown()
  }, [md])

  let content, title=''
  if (markdown) {
    // Extract comments from markdown. We are using this sytax for a comment:
    // [//]: # (Comment in here)
    // See https://www.jamestharpe.com/markdown-comments/
    const comments = markdown
      .split(/\r?\n/)
      .filter(md => md.startsWith('[//]: # ('))
      .map(c => c.substring(9, c.length-1))

    // Look for page title in comments
    comments.forEach(c => {
      if (c.startsWith('title: ')) {
        title = c.substring(7)
      }
    })
    content = <ReactMarkdown escapeHtml={false} source={markdown} />
  } else {
    title = 'Content not found'
    content = (
      <>
        <p>
          The markdown page referenced cannot be found: <b>{md}</b>
        </p>
      </>
    )
  }
  return (
    <IonPage className="webPage">
      <Track/>
      <IonHeader>
        <IonToolbar>
          <IonButtons slot="start">
            <IonMenuButton />
          </IonButtons>
          <IonTitle>{title}</IonTitle>
        </IonToolbar>
      </IonHeader>
      <IonContent className='infoContent'>
        {content}
        {/* <IonGrid>
          <IonRow>
            <IonCol size="12" size-xl="8" className="ion-padding">
              {content}
            </IonCol>
            <IonCol size="12" size-xl="4" className="ion-padding">
              <SocialMediaFeed/>
            </IonCol>
          </IonRow>
        </IonGrid> */}
      </IonContent>
    </IonPage>
  )
}

export default WebPage