import Koa from 'koa'
import Router from 'koa-router'
import logger from 'koa-logger'
import serve from 'koa-static'
import body from 'koa-body'
import cors from '@koa/cors'
import { getUser, insertUser, updateUser } from './user.js'

global.devUser = ''

const app = new Koa()

// For servicing API requests
const router = new Router()

function getUsername(ctx) {
    if(ctx.request.header.host.substr(0,9) === 'localhost') {
        // For testing on localhost, username will be 
        // set in global variable devUser.
        return global.devUser
    } else {
        // On production and staging the username is inserted
        // into header.remote-user by the Crowd SSO proxy server.
        return ctx.request.header['remote-user']
    }
}

router
   .get('/db/setdevuser/:username', (ctx) => {
    global.devUser = ctx.params.username
    ctx.body = {username: global.devUser}
})

router
   .get('/db/cleardevuser', (ctx) => {
    global.devUser = ''
    ctx.body = {username: global.devUser}
})

router
   .get('/db/username', (ctx) => {
    ctx.body = {username: getUsername(ctx)}
})

router
   .get('/db/user', async (ctx) => {
    await getUser(ctx, getUsername(ctx))
})

router
   .get('/db/insert', async (ctx) => {
    await insertUser(ctx, getUsername(ctx))
})

router
   .post('/db/update', async (ctx) => {
    await updateUser(ctx, getUsername(ctx))
})

app
    .use(cors({origin: '*'})) // Has to be above the serving options
    .use(serve('public'))
    .use(serve('/static')) 
    .use(body())
    .use(logger())
    .use(router.routes())
    .use(router.allowedMethods())
    .listen(80)