import sqlite3 from 'sqlite3'
import { open } from 'sqlite'
import * as crypto from 'crypto'

sqlite3.verbose()

const prefs = [
  {
    name: 'username',
    column: 'txtvalue'
  },
  {
    name: 'firstname',
    column: 'txtvalue'
  },
  {
    name: 'fullname',
    column: 'txtvalue'
  },
  {
    name: 'usedata',
    column: 'intvalue'
  },
  {
    name: 'moths',
    column: 'intvalue'
  },
  {
    name: 'butterflies',
    column: 'intvalue'
  },
  {
    name: 'grasshoppers',
    column: 'intvalue'
  },
  {
    name: 'generalConsent',
    column: 'intvalue'
  },
  {
    name: 'commsConsent',
    column: 'txtvalue'
  },
  {
    name: 'otherPlatformConsent',
    column: 'intvalue'
  },
  {
    name: 'ispotUsername',
    column: 'txtvalue'
  },
  {
    name: 'irecordUsername',
    column: 'txtvalue'
  }
]

const formatResponse = (data) => {
  const response = {}
  data.forEach(d => {
    const pref = prefs.find(({name}) => name === d.name)
    if (pref) {
      response[d.name] = d[pref['column']]
    } 
  })
  return response
}

const openDb = async (username) => {
  const dbFile =  `/sqlite/db${crypto.createHash('md5').update(username).digest("hex")}.db`
  const db = await open ({filename: dbFile, driver: sqlite3.Database})
  await db.run(`CREATE TABLE IF NOT EXISTS preferences (
    name TEXT PRIMARY KEY,
    txtvalue TEXT,
    intvalue INTEGER,
    realvalue REAL,
    blobvalue BLOB
  );`)
  return db
}

const updatePreference = async (db, ctx, name, column) => {
  if (ctx.request.body[name] !== null && ctx.request.body[name] !== undefined) {
    return await db.run(`INSERT INTO preferences(name, ${column}) VALUES('${name}', '${ctx.request.body[name]}') 
    ON CONFLICT(name) DO UPDATE SET ${column}='${ctx.request.body[name]}'`)
  }
}

const getUser = async (ctx, username) => {

  console.log("Get user preferences")
  const db = await openDb(username)
  ctx.body = formatResponse(await db.all('SELECT * from preferences'))
  db.close()
}

const insertUser = async (ctx, username) => {
 
  console.log("Creating new user in users db")
  const db = await openDb(username)
  await db.run(`INSERT INTO preferences (name, txtvalue) VALUES('username', '${username}')`)
  ctx.body = formatResponse(await db.all('SELECT * from preferences'))
  db.close()
}

const updateUser = async (ctx, username) => {

  console.log("Updating user preferences")
  const db = await openDb(username)
  for (const p of prefs) {
    await updatePreference(db, ctx, p.name, p.column)
  }
  ctx.body = formatResponse(await db.all('SELECT * from preferences'))
  db.close()
}

export { getUser, insertUser, updateUser }